{
  "type": "record",
  "Aspect": {
    "name": "queryUsageFeatures"
  },
  "name": "QueryUsageFeatures",
  "namespace": "com.linkedin.pegasus2avro.query",
  "fields": [
    {
      "Searchable": {
        "fieldName": "queryCountLast30DaysFeature",
        "fieldType": "COUNT"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "queryCountLast30Days",
      "default": null,
      "doc": "Number of times this query was executed in the last 30 days"
    },
    {
      "Searchable": {
        "fieldName": "queryCountFeature",
        "fieldType": "COUNT"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "queryCountTotal",
      "default": null,
      "doc": "Query count total \nTotal number of executions of this query"
    },
    {
      "Searchable": {
        "/*": {
          "fieldName": "topUsersLast30DaysFeature",
          "fieldType": "URN",
          "hasValuesFieldName": "hasTopUsersLast30DaysFeature"
        }
      },
      "type": [
        "null",
        {
          "type": "array",
          "items": "string"
        }
      ],
      "name": "topUsersLast30Days",
      "default": null,
      "doc": "Top users of this query in the last 30 days",
      "Urn": "Urn",
      "urn_is_array": true
    },
    {
      "Searchable": {
        "fieldName": "queryCostLast30DaysFeature"
      },
      "type": [
        "null",
        "double"
      ],
      "name": "queryCostLast30Days",
      "default": null,
      "doc": "Total cost of this query in the last 30 days"
    },
    {
      "Searchable": {
        "fieldName": "runsPercentileLast30days"
      },
      "type": [
        "null",
        "int"
      ],
      "name": "runsPercentileLast30days",
      "default": null,
      "doc": "The percentile rank of this query based on runs last 30 days"
    },
    {
      "Searchable": {
        "fieldName": "lastExecutedAtFeature",
        "fieldType": "DATETIME"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "lastExecutedAt",
      "default": null,
      "doc": "Last time this query was executed"
    }
  ]
}