{
  "type": "record",
  "Aspect": {
    "name": "anomalyKey"
  },
  "name": "AnomalyKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "A unique id for the anomaly. Generated on the server side at anomaly creation time."
    }
  ],
  "doc": "Key for an asset Anomaly"
}